package yatzy;

import java.util.ArrayList;
import java.util.List;

/**
 * Used to notify the GUI of changes in model. Possible send some data to GUI
 * for display (if not, use null as argument)
 *
 * Class a bit complicated but you don't need to understand every 
 * detail to use it
 * 
 * @author hajo
 */
// enum is very much like a class
public enum EventBus {

    INSTANCE;// This is the one and only instance of EventBus (named INSTANCE)

    // This is an (inner) interface. A specification.
    public interface IEventHandler {
        // All GUI parts that wish to receive event must implement this

        public void onEvent(Event evt, Object o);
    };

    public enum Event {  // All possible events

        DISABLE_DICE,
        ENABLE_DICE,
        ENABLE_ROLL,
        DISABLE_ROLL,
        NEXT_PLAYER,
        UPDATE_SCORE,
        UPDATE_DICE,
        REJECTED_VALUE,
        ACCEPTED_VALUE,
        UPDATE_POSSIBILITIES,
        FINISHED
    };
    private boolean trace = true;
    // A list of listeners
    private final List<IEventHandler> handlers = new ArrayList<>();

    // A listener can register
    public void register(IEventHandler handler) {
        handlers.add(handler);
    }
    
    // A listener can unregister
    public void unRegister(IEventHandler handler) {
        handlers.remove(handler);
    }

    // Called by any class that would like to publish
    // an event to all registred listeners
    public void publish(Event evt, Object data) {
        // Traceing all events is extremely helpful during development
        if (trace) {
            System.out.println(evt);
        }
        for (IEventHandler evh : handlers) {
            evh.onEvent(evt, data);
        }
    }

    // Enable/disable printing out events
    public void setTrace(boolean b) {
        this.trace = b;
    }
}
