package yatzy.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import yatzy.YatzyOptions;
import yatzy.Player;

/**
 * Dialog to set options
 * @author hajo
 */
public class PlayersDialog extends JDialog implements ActionListener {

    private JCheckBox robot;
    private List<JTextField> players = new ArrayList<>();

    PlayersDialog(JFrame parent) {
        this.setTitle("Players");
        this.setLocationRelativeTo(parent);
        this.setLayout(new BorderLayout());
        this.add(createPlayersPanel(), BorderLayout.NORTH);
        this.add(createRobotPanel(), BorderLayout.CENTER);
        this.add(createButtonPanel(), BorderLayout.SOUTH);
        this.addWindowListener(windowListener);
        this.pack();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        switch (e.getActionCommand()) {
            case "ok":
                List<Player> ps = new ArrayList<>();
                int ordinal = 0;
                for (JTextField t : players) {
                    if (t.getText().length() > 0) {
                        ps.add(new Player(t.getText(), ordinal));
                        ordinal++;
                    }
                }
                YatzyOptions.setPlayers(ps);
                YatzyOptions.setRobot(robot.isSelected());
            case "cancel":
                break;
            default:
                ;
        }
        this.dispose();
    }

    private JPanel createPlayersPanel() {
        JPanel p = new JPanel();
        p.setLayout(new GridLayout(YatzyOptions.MAX_PLAYERS, 2, 2, 2));
        p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        for (int i = 0; i < YatzyOptions.MAX_PLAYERS; i++) {
            JLabel l = new JLabel("Player " + (i + 1));
            p.add(l);
            JTextField t = new JTextField(20);
            t.setBorder(BorderFactory.createLineBorder(Color.GRAY));
            players.add(t);
            p.add(t);
        }
        return p;
    }

    private JPanel createRobotPanel() {
        JPanel p = new JPanel();
        p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        p.setLayout(new GridLayout(1, 2, 2, 2));       
        JLabel l = new JLabel("Use Robot");
        p.add(l);
        robot = new JCheckBox("Yes");
        p.add(robot);
        return p;
    }

    private JPanel createButtonPanel() {
        JPanel p = new JPanel();
         p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        JButton ok = new JButton("Ok");
        ok.setActionCommand("ok");
        JButton cancel = new JButton("Cancel");
        cancel.setActionCommand("cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        p.add(ok);
        p.add(cancel);
        return p;
    }
    
    WindowAdapter windowListener = new WindowAdapter() {
        @Override
        public void windowOpened(WindowEvent e) {
            List<Player> ps = YatzyOptions.getPlayers();
            int i = 0;
            for (Player p : ps) {
                players.get(i).setText(p.getName());
                i++;
            }
        }
    };
}