/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package yatzy.gui.fancy;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.SwingWorker;
import yatzy.EventBus;
import yatzy.Yatzy;

/**
 * No inherit from DicePanel too much differ
 *
 * @author hajo
 */
public class FancyDicePanel extends FancyPanel implements ActionListener, EventBus.IEventHandler {

    private final Yatzy yatzy;
    private final String path = "/yatzy/gui/fancy/";
    private ImageIcon blurred;
    private ImageIcon black;
    Map<Integer, ImageIcon> valueImage = new HashMap<>();
    AnimatedDiceButton[] btns = new AnimatedDiceButton[5];

    public FancyDicePanel(Yatzy yatzy) {
        this.yatzy = yatzy;
       
        URL imgURL = getClass().getResource(path + "dice6blurred.png");
        blurred = new ImageIcon(imgURL, "blurred");
        imgURL = getClass().getResource(path + "dice4_blacksmall.png");
        black = new ImageIcon(imgURL, "black");

        for (int i = 1; i <= 6; i++) {
            String fileName = "dice" + i + "small.png";
            imgURL = getClass().getResource(path + fileName);
            ImageIcon ii = new ImageIcon(imgURL, "dice" + i);
            valueImage.put(i, ii);
        }
        for (int i = 0; i < 5; i++) {
            btns[i] = new AnimatedDiceButton(100, 1000, blurred, valueImage.get(5), black);
            btns[i].setActionCommand("toggle");
            btns[i].setBorder(null);
            btns[i].addActionListener(this);
            this.add(btns[i]);
        }
        this.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        EventBus.INSTANCE.register(this);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        System.out.println(action);
        switch (action) {
            case "toggle":
                AnimatedDiceButton b = (AnimatedDiceButton) e.getSource();
                int i = getIndex(b);
                if (b.isSelected()) {
                    b.setBorder(BorderFactory.createLineBorder(Color.RED, 3));
                    //System.out.println("disabled");
                    yatzy.disableDice(i);
                } else {
                    b.setBorder(null);
                    //System.out.println("enabled");
                    yatzy.enableDice(i);
                }
                break;
            default:
                ;
        }
    }

    @Override
    public void onEvent(EventBus.Event evt, Object o) {
        if (evt == EventBus.Event.UPDATE_DICE) {
            List<Integer> result = (List<Integer>) o;
            updateDice(result);
        } else if (evt == EventBus.Event.NEXT_PLAYER) {
            nextPlayer();
        }
    }

    private void nextPlayer() {
        for (Component c : this.getComponents()) {
            AnimatedDiceButton b = (AnimatedDiceButton) c;
            b.setSelected(false);
            b.setStaticIcon(blurred);
            b.setBorder(null);
            this.validate(); // Must have both (in this order?)
            this.repaint();
        }
    }

    private void updateDice(final List<Integer> result) {
        // Must have this to let animation finish 
        // before displaying the possibilities (RobotYazty)
        // ALSO : Can't block Swing thread!
        SwingWorker worker = new SwingWorker<Void, Object>() {
            private boolean isAnimating() {
                boolean isAnimating = false;
                for (AnimatedDiceButton b : btns) {
                    if (b.isAnimating()) {
                        isAnimating = true;
                    }
                }
                return isAnimating;
            }

            @Override
            public Void doInBackground() {
                int i = 0;
                for (AnimatedDiceButton b : btns) {
                    b.start();
                    b.setStaticIcon(valueImage.get(result.get(i)));
                    i++;
                }
                while (isAnimating()) {
                }
                return null;
            }

            @Override
            protected void done() {
                yatzy.getPossibilities();
            }
        };
        worker.execute();
    }

    private int getIndex(Component component) {
        int i = 0;
        for (Component c : this.getComponents()) {
            if (c == component) {
                return i;
            }
            i++;
        }
        return -1;
    }
}
